using System;
using System.Configuration;
using System.Data;
using System.Text.RegularExpressions;
using System.Web;
using System.Web.Configuration;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;

namespace ProjectName
{
    public class WpfBrowserCapabilites
    {
        const string WpfVersion1FinalRetailBuildNumber = "3.0.04506";
        const double WindowsVistaVersionNumber = 6.0d;
        const double WindowsXPVersionNumber = 5.1d;

        static Regex WindowsVersionExpression = new Regex(@"Windows NT ([0-9]\.[0-9])");
        static Version wpfV1FinalVersion = new Version(WpfVersion1FinalRetailBuildNumber);
        
        HttpCapabilitiesBase httpCapabilities;
        string userAgent;

        // VISTA RTM + Infopath - "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.0.04506; InfoPath.2)"
        // Firefox 2.0 on Vista RTM - "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.8.1.2) Gecko/20070219 Firefox/2.0.0.2"
        // Firefox 1.5 on XP SP2 - Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.0.9) Gecko/20061206 Firefox/1.5.0.9
        // IE7 on XP SP2 - "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30)"
        // for an exhaustive list of user agent strings see here: http://en.wikipedia.org/wiki/User_agent 

        public WpfBrowserCapabilites()
        {
            this.httpCapabilities = HttpCapabilitiesBase.GetConfigCapabilities("system.web/browserCaps", HttpContext.Current.Request);
            userAgent = HttpContext.Current.Request.UserAgent;
        }

        public bool IsWPFAvailable
        {
            get
            {
                if (this.IsIEBrowser)
                {
                    Version[] clrVersions = httpCapabilities.GetClrVersions();

                    foreach (Version clrVersion in clrVersions)
                    {
                        if (clrVersion >= wpfV1FinalVersion)
                        {
                            return true;
                        }
                    }

                    return false;
                }
                else
                {
                    return this.IsWindowsVersionWithWPFPreInstalled;
                }
            }
        }

        public bool WindowsClientVersionCanSupportWpf
        {
            get
            {
                double windowsVersionNumber = GetWindowsClientVersion();
                return (windowsVersionNumber >= WindowsXPVersionNumber);
            }
        }

        public bool IsWindowsClient
        {
            get
            {
                return httpCapabilities.Win32;
            } 
        }

        public bool IsIEBrowser
        {
            get
            {
                return string.Compare(httpCapabilities.Browser, "IE", true) == 0;
            }
        }

        private bool IsWindowsVersionWithWPFPreInstalled
        {
            get
            {
                double windowsVersionNumber = GetWindowsClientVersion();
                return (windowsVersionNumber >= WindowsVistaVersionNumber);
            }
        }

        private double GetWindowsClientVersion()
        {
            double versionNumber = 0;

            if (WindowsVersionExpression.IsMatch(userAgent))
            {
                Match windowsVersionMatch = WindowsVersionExpression.Match(userAgent);
                string versionNumberString = windowsVersionMatch.Groups[1].Captures[0].Value;
                double.TryParse(versionNumberString, out versionNumber);
            }
            return versionNumber;
        }
    }
}
